# -*- coding: utf-8 -*-

import os
import requests
import copy
import re

import pytest
import shutil

from academic_tracker.ref_srch_modularized import input_reading_and_checking, build_publication_dict, save_and_send_reports_and_emails
from academic_tracker.fileio import load_json, read_text_from_txt


@pytest.fixture(autouse=True)
def disable_network_calls(monkeypatch):
    def stunted_get():
        raise RuntimeError("Network access not allowed during testing!")
    monkeypatch.setattr(requests, "get", lambda *args, **kwargs: stunted_get())



@pytest.fixture
def config_dict():
    return load_json(os.path.join("tests", "testing_files", "config_truncated.json"))


def test_input_reading_and_checking_no_prev_pub(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated.json")
    ref_path_or_URL = os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json")
    
    expected_config_dict = config_dict
    
    actual_config_dict, tokenized_citations, has_previous_pubs, prev_pubs = input_reading_and_checking(config_json_filepath, ref_path_or_URL, False, False, False, "ignore")
    
    assert expected_config_dict == actual_config_dict
    assert tokenized_citations == load_json(os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json"))
    assert has_previous_pubs == False
    assert prev_pubs == {}



def test_input_reading_and_checking_has_prev_pub(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated.json")
    ref_path_or_URL = os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json")
    prev_pub_filepath = os.path.join("tests", "testing_files", "publication_dict_truncated.json")
    
    expected_config_dict = config_dict
    
    actual_config_dict, tokenized_citations, has_previous_pubs, prev_pubs = input_reading_and_checking(config_json_filepath, ref_path_or_URL, False, False, False, prev_pub_filepath)
    
    assert expected_config_dict == actual_config_dict
    assert tokenized_citations == load_json(os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json"))
    assert has_previous_pubs == True
    assert prev_pubs == load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    
    
def test_input_reading_and_checking_noCrossref(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated_noCrossref.json")
    ref_path_or_URL = os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json")
    
    expected_config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated_noCrossref.json"))
    
    actual_config_dict, tokenized_citations, has_previous_pubs, prev_pubs = input_reading_and_checking(config_json_filepath, ref_path_or_URL, False, False, False, "ignore")
    
    assert expected_config_dict == actual_config_dict
    assert tokenized_citations == load_json(os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json"))
    assert has_previous_pubs == False
    assert prev_pubs == {}
    

def test_input_reading_and_checking_noPubMed(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated_noPubMed.json")
    ref_path_or_URL = os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json")
    
    expected_config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated_noPubMed.json"))
    
    actual_config_dict, tokenized_citations, has_previous_pubs, prev_pubs = input_reading_and_checking(config_json_filepath, ref_path_or_URL, False, False, False, "ignore")
    
    assert expected_config_dict == actual_config_dict
    assert tokenized_citations == load_json(os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json"))
    assert has_previous_pubs == False
    assert prev_pubs == {}




def test_build_publication_dict_with_Crossref(config_dict, mocker):
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    
    input_tokenized_citations = copy.deepcopy(expected_tokenized_citations)
    for citation in input_tokenized_citations:
        citation["pub_dict_key"] = ""
    
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    
    def mock_query(*args, **kwargs):
        return {"https://doi.org/10.3390/metabo10090368":expected_pub_dict["https://doi.org/10.3390/metabo10090368"]}, [None, "https://doi.org/10.3390/metabo10090368"]
    mocker.patch("academic_tracker.ref_srch_modularized.ref_srch_webio.search_references_on_PubMed", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.3390/metabo11030163":expected_pub_dict["https://doi.org/10.3390/metabo11030163"]}, ["https://doi.org/10.3390/metabo11030163", None]
    mocker.patch("academic_tracker.ref_srch_modularized.ref_srch_webio.search_references_on_Crossref", mock_query2)
    
    actual_publication_dict, actual_tokenized_citations = build_publication_dict(config_dict, input_tokenized_citations, False, False)
    
    assert expected_pub_dict == actual_publication_dict
    assert expected_tokenized_citations == actual_tokenized_citations
    


def test_build_publication_dict_no_Crossref(config_dict, mocker):
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    expected_tokenized_citations[0]["pub_dict_key"] = ""
    
    input_tokenized_citations = copy.deepcopy(expected_tokenized_citations)
    for citation in input_tokenized_citations:
        citation["pub_dict_key"] = ""
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    del expected_pub_dict["https://doi.org/10.3390/metabo11030163"]
    
    def mock_query(*args, **kwargs):
        return {"https://doi.org/10.3390/metabo10090368":expected_pub_dict["https://doi.org/10.3390/metabo10090368"]}, [None, "https://doi.org/10.3390/metabo10090368"]
    mocker.patch("academic_tracker.ref_srch_modularized.ref_srch_webio.search_references_on_PubMed", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.3390/metabo11030163":expected_pub_dict["https://doi.org/10.3390/metabo11030163"]}, ["https://doi.org/10.3390/metabo11030163", None]
    mocker.patch("academic_tracker.ref_srch_modularized.ref_srch_webio.search_references_on_Crossref", mock_query2)
        
    actual_publication_dict, actual_tokenized_citations = build_publication_dict(config_dict, input_tokenized_citations, True, False)
    
    assert expected_pub_dict == actual_publication_dict
    assert expected_tokenized_citations == actual_tokenized_citations
    


def test_build_publication_dict_no_PubMed(config_dict, mocker):
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    expected_tokenized_citations[1]["pub_dict_key"] = ""
    
    input_tokenized_citations = copy.deepcopy(expected_tokenized_citations)
    for citation in input_tokenized_citations:
        citation["pub_dict_key"] = ""
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    del expected_pub_dict["https://doi.org/10.3390/metabo10090368"]
    
    def mock_query(*args, **kwargs):
        return {"https://doi.org/10.3390/metabo10090368":expected_pub_dict["https://doi.org/10.3390/metabo10090368"]}, [None, "https://doi.org/10.3390/metabo10090368"]
    mocker.patch("academic_tracker.ref_srch_modularized.ref_srch_webio.search_references_on_PubMed", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.3390/metabo11030163":expected_pub_dict["https://doi.org/10.3390/metabo11030163"]}, ["https://doi.org/10.3390/metabo11030163", None]
    mocker.patch("academic_tracker.ref_srch_modularized.ref_srch_webio.search_references_on_Crossref", mock_query2)
        
    actual_publication_dict, actual_tokenized_citations = build_publication_dict(config_dict, input_tokenized_citations, False, True)
    
    assert expected_pub_dict == actual_publication_dict
    assert expected_tokenized_citations == actual_tokenized_citations




def test_save_and_send_reports_and_emails_no_email(config_dict):
    
    tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["template"] = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_template_string.txt"))
    
    save_dir = save_and_send_reports_and_emails(config_dict, tokenized_citations, pub_dict, {}, False, False)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_test1.txt")) == read_text_from_txt(os.path.join(save_dir, "summary_report.txt"))
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))



def test_save_and_send_reports_and_emails_with_email(config_dict):
    
    tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["template"] = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_template_string.txt"))
    config_dict["summary_report"]["from_email"] = "ptth222@uky.edu"
    config_dict["summary_report"]["to_email"] = ["ptth222@uky.edu"]
    config_dict["summary_report"]["cc_email"] = ["ptth222@uky.edu"]
    config_dict["summary_report"]["email_body"] = "Body"
    config_dict["summary_report"]["email_subject"] = "Subject"
    
    save_dir = save_and_send_reports_and_emails(config_dict, tokenized_citations, pub_dict, {}, False, True)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_test1.txt")) == read_text_from_txt(os.path.join(save_dir, "summary_report.txt"))
    assert os.path.exists(os.path.join(save_dir, "emails.json"))



def test_save_and_send_reports_and_emails_default_template(config_dict):
    
    tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    
    config_dict["summary_report"] = {}
    
    save_dir = save_and_send_reports_and_emails(config_dict, tokenized_citations, pub_dict, {}, False, False)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_default.txt")) == read_text_from_txt(os.path.join(save_dir, "summary_report.txt"))
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))
    
    
def test_save_and_send_reports_and_emails_tabular(config_dict):
    
    tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["columns"] = {"Col1":"<author_first>"}
    
    save_dir = save_and_send_reports_and_emails(config_dict, tokenized_citations, pub_dict, {}, False, False)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.csv"))
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))
    
    
def test_save_and_send_reports_and_emails_manual_name(config_dict):
    
    tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
    pub_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["template"] = read_text_from_txt(os.path.join("tests", "testing_files", "ref_srch_report_template_string.txt"))
    config_dict["summary_report"]["filename"] = "test_name.txt"
    
    save_dir = save_and_send_reports_and_emails(config_dict, tokenized_citations, pub_dict, {}, False, False)
    
    assert os.path.exists(os.path.join(save_dir, "test_name.txt"))
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))









@pytest.fixture(autouse=True)
def cleanup(request):
    """Cleanup a testing directory once we are finished."""
    def remove_test_dir():
        dir_contents = os.listdir()
        tracker_dirs = [folder for folder in dir_contents if re.match(r"tracker-(\d{10})", folder) or re.match(r"tracker-test-(\d{10})", folder)]
        for directory in tracker_dirs:
            shutil.rmtree(directory)
    request.addfinalizer(remove_test_dir)








# -*- coding: utf-8 -*-



import os
import time
from datetime import datetime
import re
import shutil

import pytest
import pandas

from academic_tracker.fileio import load_json, read_previous_publications, save_publications_to_file, save_emails_to_file, read_text_from_txt 
from academic_tracker.fileio import read_text_from_docx, read_csv, save_string_to_file, save_json_to_file
from fixtures import email_messages


TESTING_DIR = "test_dir"


@pytest.fixture
def test_json():
    return {
              "affiliations": [
                "kentucky"
              ],
              "cc_email": [],
              "cutoff_year": 2019,
              "email_subject": "New PubMed Publications",
              "email_template": "Hey <author_first_name>,\n\nThese are the publications I was able to find on PubMed. Are any missing?\n\n<total_pubs>\n\nKind regards,\n\nThis email was sent by an automated service. If you have any questions or concerns please email my creator ptth222@uky.edu",
              "from_email": "ptth222@uky.edu",
              "grants": [
                "P42ES007380",
                "P42 ES007380"
              ]
            }


def test_load_json_error():
    path = os.path.join("tests", "testing_files", "load_json_error")
    
    with pytest.raises(BaseException):
        load_json(path)


def test_load_json_no_error(test_json):
    
    path = os.path.join("tests", "testing_files", "config.json")
    
    data = load_json(path)
    
    assert data == test_json
    
    
def test_load_json_no_path():
    path = os.path.join("tests", "testing_files", "asdf.asdf")
    
    with pytest.raises(SystemExit):
        load_json(path)



def test_read_text_from_txt_error():
    path = os.path.join("tests", "testing_files", "load_json_error")
    
    with pytest.raises(BaseException):
        read_text_from_txt(path)


def test_read_text_from_txt_no_error():
    
    path = os.path.join("tests", "testing_files", "testing_text.txt")
    
    data = read_text_from_txt(path)
    
    assert data == 'line 1\nline 2'
    

def test_read_text_from_txt_no_path():
    path = os.path.join("tests", "testing_files", "asdf.asdf")
    
    with pytest.raises(SystemExit):
        read_text_from_txt(path)
    


def test_read_text_from_docx_error():
    path = os.path.join("tests", "testing_files", "load_json_error")
    
    with pytest.raises(BaseException):
        read_text_from_docx(path)


def test_read_text_from_docx_no_error():
    
    path = os.path.join("tests", "testing_files", "testing_docx.docx")
    
    data = read_text_from_docx(path)
    
    assert data == 'Line 1\nLine 2'
    
    
def test_read_text_from_docx_no_path():
    path = os.path.join("tests", "testing_files", "asdf.asdf")
    
    with pytest.raises(SystemExit):
        read_text_from_docx(path)
    


def test_read_csv_error():
    path = os.path.join("tests", "testing_files", "load_json_error")
    
    with pytest.raises(BaseException):
        read_csv(path)


def test_read_csv_no_error():
    
    path = os.path.join("tests", "testing_files", "testing_csv.csv")
    
    data = read_csv(path)
    
    assert data.equals(pandas.DataFrame({"col1":["data1"], "col2":["data2"]}))
    

def test_read_csv_no_path():
    path = os.path.join("tests", "testing_files", "asdf.asdf")
    
    with pytest.raises(SystemExit):
        read_csv(path)



@pytest.fixture
def tracker_latest_dir():
    dir_name = "tracker-9999999999"
    os.mkdir(dir_name)
    yield
    os.rmdir(dir_name)


@pytest.fixture
def tracker_today_dir():
    save_dir_name = "tracker-" + re.sub(r"\-| |\:", "", str(datetime.now())[2:16])
    publication_save_path = os.path.join(save_dir_name, "publications.json")
    os.mkdir(save_dir_name)
    time_to_wait = 10
    time_counter = 0
    
    while not os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_dir_name + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    save_publications_to_file(save_dir_name, {}, {"test":"test"})
    time_counter = 0
    while not os.path.exists(publication_save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(publication_save_path + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    yield
    
    os.remove(publication_save_path)
    time_counter = 0
    while os.path.exists(publication_save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(publication_save_path + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    os.rmdir(save_dir_name)
    time_counter = 0
    while os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(save_dir_name + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")


    

def test_read_previous_publications_no_publications():
    
    has_previous_pubs, prev_pubs = read_previous_publications(None)
    
    assert not has_previous_pubs and prev_pubs == {}


def test_read_previous_publications_ignore():
    
    has_previous_pubs, prev_pubs = read_previous_publications("ignore")
    
    assert not has_previous_pubs and prev_pubs == {}
    
    
def test_read_previous_publications_read_until(tracker_latest_dir, tracker_today_dir):
    
    has_previous_pubs, prev_pubs = read_previous_publications(None)
    
    assert has_previous_pubs and prev_pubs == {"test":"test"}


def test_read_previous_publications_path_in_args(test_json):
    
    path = os.path.join("tests", "testing_files", "config.json")
    
    has_previous_pubs, prev_pubs = read_previous_publications(path)
    
    assert has_previous_pubs and prev_pubs == test_json







@pytest.fixture
def test_email_dir():
    save_dir_name = TESTING_DIR
    json_email_save_path = os.path.join(save_dir_name, "emails.json")
    os.mkdir(save_dir_name)
    time_to_wait = 10
    time_counter = 0
    
    while not os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_dir_name + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    yield save_dir_name, json_email_save_path
    
    os.remove(json_email_save_path)
    time_counter = 0
    while os.path.exists(json_email_save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(json_email_save_path + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
            
    
    os.rmdir(save_dir_name)
    time_counter = 0
    while os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(save_dir_name + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")






def test_save_emails_to_file_successful_save(test_email_dir, email_messages):
    
    save_dir_name, json_email_save_path = test_email_dir
    save_emails_to_file(email_messages, save_dir_name)
    
    time_to_wait = 10
    time_counter = 0
    while not os.path.exists(json_email_save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(json_email_save_path + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
            
    emails_json = load_json(json_email_save_path)
    
    del emails_json["creation_date"]
    del email_messages["creation_date"]

    assert emails_json == email_messages






@pytest.fixture
def test_pub_dir():
    save_dir_name = TESTING_DIR
    pub_save_path = os.path.join(save_dir_name, "publications.json")
    os.mkdir(save_dir_name)
    time_to_wait = 10
    time_counter = 0
    
    while not os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_dir_name + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    yield save_dir_name, pub_save_path
    
    os.remove(pub_save_path)
    time_counter = 0
    while os.path.exists(pub_save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(pub_save_path + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    os.rmdir(save_dir_name)
    time_counter = 0
    while os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(save_dir_name + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")




def test_save_publications_to_file_successful_save(test_pub_dir):
    
    save_dir_name, pub_save_path = test_pub_dir
    save_publications_to_file(save_dir_name, {}, {})
    
    time_to_wait = 10
    time_counter = 0
    while not os.path.exists(pub_save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(pub_save_path + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")

    publications_json = load_json(pub_save_path)

    assert publications_json == {}




@pytest.fixture
def test_save_string_dir():
    save_dir_name = TESTING_DIR
    filename = "test.txt"
    save_path = os.path.join(save_dir_name, filename)
    os.mkdir(save_dir_name)
    time_to_wait = 10
    time_counter = 0
    
    while not os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_dir_name + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    yield save_dir_name, save_path, filename
    
    os.remove(save_path)
    time_counter = 0
    while os.path.exists(save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(save_path + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    os.rmdir(save_dir_name)
    time_counter = 0
    while os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(save_dir_name + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")



def test_save_string_to_file_successful_save(test_save_string_dir):
    
    save_dir_name, save_path, filename = test_save_string_dir
    save_string_to_file(save_dir_name, filename, "test")
    
    time_to_wait = 10
    time_counter = 0
    while not os.path.exists(save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_path + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")

    saved_text = read_text_from_txt(save_path)

    assert saved_text == "test"




@pytest.fixture
def test_save_json_dir():
    save_dir_name = TESTING_DIR
    filename = "test.json"
    save_path = os.path.join(save_dir_name, filename)
    os.mkdir(save_dir_name)
    time_to_wait = 10
    time_counter = 0
    
    while not os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_dir_name + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    yield save_dir_name, save_path, filename
    
    os.remove(save_path)
    time_counter = 0
    while os.path.exists(save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(save_path + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")
    
    os.rmdir(save_dir_name)
    time_counter = 0
    while os.path.exists(save_dir_name):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileExistsError(save_dir_name + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")



def test_save_json_to_file_successful_save(test_save_json_dir):
    
    save_dir_name, save_path, filename = test_save_json_dir
    save_json_to_file(save_dir_name, filename, {"test":"test"})
    
    time_to_wait = 10
    time_counter = 0
    while not os.path.exists(save_path):
        time.sleep(1)
        time_counter += 1
        if time_counter > time_to_wait:
            raise FileNotFoundError(save_path + " was not created within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")

    saved_json = load_json(save_path)

    assert saved_json == {"test":"test"}










@pytest.fixture(scope="module", autouse=True)
def cleanup(request):
    """Cleanup a testing directory once we are finished."""
    def remove_test_dir():
        if os.path.exists(TESTING_DIR):
            shutil.rmtree(TESTING_DIR)
    request.addfinalizer(remove_test_dir)




# -*- coding: utf-8 -*-

import pytest

from academic_tracker import __main__

@pytest.fixture(autouse=True)
def set_verbose_and_silent(monkeypatch):
    monkeypatch.setattr(__main__, "SILENT", False)
    monkeypatch.setattr(__main__, "VERBOSE", True)




# -*- coding: utf-8 -*-

import os

import pytest

from academic_tracker.citation_parsing import parse_text_for_citations, tokenize_Vancouver_authors, tokenize_MLA_or_Chicago_authors
from academic_tracker.citation_parsing import tokenize_APA_or_Harvard_authors, tokenize_myncbi_citations, parse_MEDLINE_format
from academic_tracker.fileio import load_json, read_text_from_txt




def test_parse_text_for_citations():
    
    text = read_text_from_txt(os.path.join("tests", "testing_files", "parse_citations_test.txt"))
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_parsing_test.json"))
    
    actual_tokenized_citations = parse_text_for_citations(text)
    
    assert expected_tokenized_citations == actual_tokenized_citations



@pytest.mark.parametrize("authors_string, authors", [
        ("last_name initials", [{"last":"last_name", "initials":"initials"}]),
        ("last_name initials et al", [{"last":"last_name", "initials":"initials"}]),
        ("last_name1 initials1, last_name2 initials2", [{"last":"last_name1", "initials":"initials1"},
                                                       {"last":"last_name2", "initials":"initials2"}]),
        ("last_name1 initials1, and last_name2 initials2", [{"last":"last_name1", "initials":"initials1"},
                                                            {"last":"last_name2", "initials":"initials2"}]),
        ("last_name1 initials1 and last_name2 initials2", [{"last":"last_name1", "initials":"initials1"},
                                                           {"last":"last_name2", "initials":"initials2"}]),
        ("last_name1 initials1 & last_name2 initials2", [{"last":"last_name1", "initials":"initials1"},
                                                           {"last":"last_name2", "initials":"initials2"}]),
        ])

def test_tokenize_Vancouver_authors(authors_string, authors):
    
    assert authors == tokenize_Vancouver_authors(authors_string)
    


@pytest.mark.parametrize("authors_string, authors", [
        ("last_name, first_name middle_name", [{"first":"first_name", "middle":"middle_name", "last":"last_name"}]),
        ("last_name, first_name", [{"first":"first_name", "middle":"", "last":"last_name"}]),
        ("last_name, first_name middle_name et al.", [{"first":"first_name", "middle":"middle_name", "last":"last_name"}]),
        ("first_name middle_name last_name", [{"first":"first_name", "middle":"middle_name", "last":"last_name"}]),
        ("last_name1, first_name1 middle_name1, first_name2 middle_name2 last_name2", [{"first":"first_name1", "middle":"middle_name1", "last":"last_name1"},
                                                                                       {"first":"first_name2", "middle":"middle_name2", "last":"last_name2"}]),
        ("last_name1, first_name1 middle_name1, and first_name2 middle_name2 last_name2", [{"first":"first_name1", "middle":"middle_name1", "last":"last_name1"},
                                                                                           {"first":"first_name2", "middle":"middle_name2", "last":"last_name2"}]),
        ("last_name1, first_name1 middle_name1 and first_name2 middle_name2 last_name2", [{"first":"first_name1", "middle":"middle_name1", "last":"last_name1"},
                                                                                          {"first":"first_name2", "middle":"middle_name2", "last":"last_name2"}]),
        ("last_name1, first_name1 middle_name1, & first_name2 middle_name2 last_name2", [{"first":"first_name1", "middle":"middle_name1", "last":"last_name1"},
                                                                                         {"first":"first_name2", "middle":"middle_name2", "last":"last_name2"}]),
        ("last_name1, first_name1 middle_name1, & last_name2", [{"first":"first_name1", "middle":"middle_name1", "last":"last_name1"},
                                                                {"first":"", "middle":"", "last":"last_name2"}]),
        ("last_name1, first_name1 middle_name1, & first_name2 last_name2", [{"first":"first_name1", "middle":"middle_name1", "last":"last_name1"},
                                                                            {"first":"first_name2", "middle":"", "last":"last_name2"}]),
        ])


def test_tokenize_MLA_or_Chicago_authors(authors_string, authors):
    
    assert authors == tokenize_MLA_or_Chicago_authors(authors_string)




@pytest.mark.parametrize("authors_string, authors", [
        ("last_name, A.B.", [{"last":"last_name", "initials":"A.B."}]),
        ("last_name, A.B. et al.", [{"last":"last_name", "initials":"A.B."}]),
        ("last_name1, A.B., last_name2, C.D.", [{"last":"last_name1", "initials":"A.B."},
                                                {"last":"last_name2", "initials":"C.D."}]),
        ("last_name1, A.B., and last_name2, C.D.", [{"last":"last_name1", "initials":"A.B."},
                                                    {"last":"last_name2", "initials":"C.D."}]),
        ("last_name1, A.B. and last_name2, C.D.", [{"last":"last_name1", "initials":"A.B."},
                                                   {"last":"last_name2", "initials":"C.D."}]),
        ("last_name1, A.B. & last_name2, C.D.", [{"last":"last_name1", "initials":"A.B."},
                                                 {"last":"last_name2", "initials":"C.D."}]),
        ("last_name1, A.B. & last_name2, C.D.,", [{"last":"last_name1", "initials":"A.B."},
                                                  {"last":"last_name2", "initials":"C.D."}]),
        ("last_name1, A.B. & last_name2", [{"last":"last_name1", "initials":"A.B."},
                                           {"last":"last_name2", "initials":""}]),
        ])

def test_tokenize_APA_or_Harvard_authors(authors_string, authors):
    
    assert authors == tokenize_APA_or_Harvard_authors(authors_string)




def test_tokenize_myncbi_citations():
    pages = load_json(os.path.join("tests", "testing_files", "myncbi_webpages.json"))
    
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_myncbi_page1.json"))
    
    actual_tokenized_citations = tokenize_myncbi_citations(pages[0])
    
    assert expected_tokenized_citations == actual_tokenized_citations




def test_parse_MEDLINE_format():
    expected_tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_MEDLINE.json"))
    
    actual_tokenized_citations = parse_MEDLINE_format(read_text_from_txt(os.path.join("tests", "testing_files", "medline.txt")))
    
    assert expected_tokenized_citations == actual_tokenized_citations










# -*- coding: utf-8 -*-


import os
import requests
import re

import pytest
import shutil

from academic_tracker.athr_srch_modularized import input_reading_and_checking, generate_internal_data_and_check_authors
from academic_tracker.athr_srch_modularized import build_publication_dict, save_and_send_reports_and_emails
from academic_tracker.fileio import load_json, read_text_from_txt


@pytest.fixture(autouse=True)
def disable_network_calls(monkeypatch):
    def stunted_get():
        raise RuntimeError("Network access not allowed during testing!")
    monkeypatch.setattr(requests, "get", lambda *args, **kwargs: stunted_get())
    


@pytest.fixture
def config_dict():
    return load_json(os.path.join("tests", "testing_files", "config_truncated.json"))




def test_input_reading_and_checking(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated.json")
    
    expected_config_dict = config_dict
    
    actual_config_dict = input_reading_and_checking(config_json_filepath, False, False, False, False)
    
    assert expected_config_dict == actual_config_dict
    

def test_input_reading_and_checking_no_ORCID(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated_noORCID.json")
    
    expected_config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated_noORCID.json"))
    
    actual_config_dict = input_reading_and_checking(config_json_filepath, False, False, False, False)
    
    assert expected_config_dict == actual_config_dict
    

def test_input_reading_and_checking_no_PubMed(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated_noPubMed.json")
    
    expected_config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated_noPubMed.json"))
    
    actual_config_dict = input_reading_and_checking(config_json_filepath, False, False, False, False)
    
    assert expected_config_dict == actual_config_dict
    
    
def test_input_reading_and_checking_no_Crossref(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated_noCrossref.json")
        
    with pytest.raises(BaseException):
        actual_config_dict = input_reading_and_checking(config_json_filepath, False, False, False, False)
        

def test_input_reading_and_checking_no_Crossref_noGS(config_dict):
    config_json_filepath = os.path.join("tests", "testing_files", "config_truncated_noCrossref.json")
        
    expected_config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated_noCrossref.json"))
    
    actual_config_dict = input_reading_and_checking(config_json_filepath, False, True, False, False)
    
    assert expected_config_dict == actual_config_dict
    



def test_generate_internal_data_and_check_authors(config_dict, capsys):
    expected_authors_by_project_dict = load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))
    expected_config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated_authors_adjusted.json"))
    
    actual_authors_by_project_dict, actual_config_dict = generate_internal_data_and_check_authors(config_dict)
    
    captured = capsys.readouterr()
    
    assert len(captured.out) > 0
    assert expected_authors_by_project_dict == actual_authors_by_project_dict
    assert expected_config_dict == actual_config_dict



def test_build_publication_dict_all_sources(config_dict, mocker):
    config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    ## Add a fourth entry that is a copy of one of the others just to have a 4th to assign to the 4 different queries.
    expected_pub_dict["32095798"] = expected_pub_dict["32095784"]
    
    def mock_query(*args, **kwargs):
        return {"32095784":expected_pub_dict["32095784"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_PubMed_for_pubs", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.1002/adhm.202101820":expected_pub_dict["https://doi.org/10.1002/adhm.202101820"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_ORCID_for_pubs", mock_query2)
    
    def mock_query3(*args, **kwargs):
        return {"https://doi.org/10.1002/advs.202101999":expected_pub_dict["https://doi.org/10.1002/advs.202101999"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Crossref_for_pubs", mock_query3)
    
    def mock_query4(*args, **kwargs):
        return {"32095798":expected_pub_dict["32095798"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Google_Scholar_for_pubs", mock_query4)
    
    actual_publication_dict = build_publication_dict(config_dict, {}, False, False, False, False)
    
    assert expected_pub_dict == actual_publication_dict



def test_build_publication_dict_no_ORCID(config_dict, mocker):
    config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    ## Add a fourth entry that is a copy of one of the others just to have a 4th to assign to the 4 different queries.
    expected_pub_dict["32095798"] = expected_pub_dict["32095784"]
    del expected_pub_dict["https://doi.org/10.1002/adhm.202101820"]
    
    def mock_query(*args, **kwargs):
        return {"32095784":expected_pub_dict["32095784"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_PubMed_for_pubs", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.1002/adhm.202101820":expected_pub_dict["https://doi.org/10.1002/adhm.202101820"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_ORCID_for_pubs", mock_query2)
    
    def mock_query3(*args, **kwargs):
        return {"https://doi.org/10.1002/advs.202101999":expected_pub_dict["https://doi.org/10.1002/advs.202101999"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Crossref_for_pubs", mock_query3)
    
    def mock_query4(*args, **kwargs):
        return {"32095798":expected_pub_dict["32095798"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Google_Scholar_for_pubs", mock_query4)
    
    actual_publication_dict = build_publication_dict(config_dict, {}, True, False, False, False)
    
    assert expected_pub_dict == actual_publication_dict
    


def test_build_publication_dict_no_Crossref(config_dict, mocker):
    config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    ## Add a fourth entry that is a copy of one of the others just to have a 4th to assign to the 4 different queries.
    expected_pub_dict["32095798"] = expected_pub_dict["32095784"]
    del expected_pub_dict["https://doi.org/10.1002/advs.202101999"]
    
    def mock_query(*args, **kwargs):
        return {"32095784":expected_pub_dict["32095784"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_PubMed_for_pubs", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.1002/adhm.202101820":expected_pub_dict["https://doi.org/10.1002/adhm.202101820"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_ORCID_for_pubs", mock_query2)
    
    def mock_query3(*args, **kwargs):
        return {"https://doi.org/10.1002/advs.202101999":expected_pub_dict["https://doi.org/10.1002/advs.202101999"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Crossref_for_pubs", mock_query3)
    
    def mock_query4(*args, **kwargs):
        return {"32095798":expected_pub_dict["32095798"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Google_Scholar_for_pubs", mock_query4)
    
    actual_publication_dict = build_publication_dict(config_dict, {}, False, False, True, False)
    
    assert expected_pub_dict == actual_publication_dict
    


def test_build_publication_dict_no_Google_Scholar(config_dict, mocker):
    config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    ## Add a fourth entry that is a copy of one of the others just to have a 4th to assign to the 4 different queries.
    expected_pub_dict["32095798"] = expected_pub_dict["32095784"]
    del expected_pub_dict["32095798"]
    
    def mock_query(*args, **kwargs):
        return {"32095784":expected_pub_dict["32095784"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_PubMed_for_pubs", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.1002/adhm.202101820":expected_pub_dict["https://doi.org/10.1002/adhm.202101820"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_ORCID_for_pubs", mock_query2)
    
    def mock_query3(*args, **kwargs):
        return {"https://doi.org/10.1002/advs.202101999":expected_pub_dict["https://doi.org/10.1002/advs.202101999"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Crossref_for_pubs", mock_query3)
    
    def mock_query4(*args, **kwargs):
        return {"32095798":expected_pub_dict["32095798"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Google_Scholar_for_pubs", mock_query4)
    
    actual_publication_dict = build_publication_dict(config_dict, {}, False, True, False, False)
    
    assert expected_pub_dict == actual_publication_dict
    
 
    
def test_build_publication_dict_no_PubMed(config_dict, mocker):
    config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    ## Add a fourth entry that is a copy of one of the others just to have a 4th to assign to the 4 different queries.
    expected_pub_dict["32095798"] = expected_pub_dict["32095784"]
    del expected_pub_dict["32095784"]
    
    def mock_query(*args, **kwargs):
        return {"32095784":expected_pub_dict["32095784"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_PubMed_for_pubs", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.1002/adhm.202101820":expected_pub_dict["https://doi.org/10.1002/adhm.202101820"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_ORCID_for_pubs", mock_query2)
    
    def mock_query3(*args, **kwargs):
        return {"https://doi.org/10.1002/advs.202101999":expected_pub_dict["https://doi.org/10.1002/advs.202101999"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Crossref_for_pubs", mock_query3)
    
    def mock_query4(*args, **kwargs):
        return {"32095798":expected_pub_dict["32095798"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Google_Scholar_for_pubs", mock_query4)
    
    actual_publication_dict = build_publication_dict(config_dict, {}, False, False, False, True)
    
    assert expected_pub_dict == actual_publication_dict
    

    
def test_build_publication_dict_duplicates(config_dict, mocker):
    config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    ## Add a fourth entry that is a copy of one of the others just to have a 4th to assign to the 4 different queries.
    expected_pub_dict["32095798"] = expected_pub_dict["32095784"]
    prev_pubs = {"32095798":expected_pub_dict["32095798"]}
    
    
    def mock_query(*args, **kwargs):
        return {"32095784":expected_pub_dict["32095784"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_PubMed_for_pubs", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {"https://doi.org/10.1002/adhm.202101820":expected_pub_dict["https://doi.org/10.1002/adhm.202101820"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_ORCID_for_pubs", mock_query2)
    
    def mock_query3(*args, **kwargs):
        return {"https://doi.org/10.1002/advs.202101999":expected_pub_dict["https://doi.org/10.1002/advs.202101999"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Crossref_for_pubs", mock_query3)
    
    def mock_query4(*args, **kwargs):
        return {"32095798":expected_pub_dict["32095798"]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Google_Scholar_for_pubs", mock_query4)
    
    actual_publication_dict = build_publication_dict(config_dict, prev_pubs, False, False, False, False)
    
    del expected_pub_dict["32095798"]
    
    assert expected_pub_dict == actual_publication_dict



def test_build_publication_dict_no_pubs_found(config_dict, mocker, capsys):
    config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
           
    def mock_query(*args, **kwargs):
        return {}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_PubMed_for_pubs", mock_query)
    
    def mock_query2(*args, **kwargs):
        return {}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_ORCID_for_pubs", mock_query2)
    
    def mock_query3(*args, **kwargs):
        return {}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Crossref_for_pubs", mock_query3)
    
    def mock_query4(*args, **kwargs):
        return {}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_webio.search_Google_Scholar_for_pubs", mock_query4)
    
    with pytest.raises(SystemExit):
        actual_publication_dict = build_publication_dict(config_dict, {}, False, False, False, False)
    
    captured = capsys.readouterr()
    expected_message = "No new publications found." + "\n"
    assert expected_message in captured.out
    



def test_save_and_send_reports_and_emails_no_email(config_dict, mocker):
    
    def emails(*args, **kwargs):
        return {"creation_date":"asdf", "emails":[]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_emails_and_reports.create_project_reports_and_emails", emails)
    
    pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    authors_by_project_dict = load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["template"] = read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_build_loop_template_string.txt"))
    
    save_dir = save_and_send_reports_and_emails(authors_by_project_dict, pub_dict, config_dict, True)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_summary_report_custom_template.txt")) == read_text_from_txt(os.path.join(save_dir, "summary_report.txt"))
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))



def test_save_and_send_reports_and_emails_with_email(config_dict, mocker):
    
    def emails(*args, **kwargs):
        return {"creation_date":"asdf", "emails":[]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_emails_and_reports.create_project_reports_and_emails", emails)
    
    pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    authors_by_project_dict = load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["template"] = read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_build_loop_template_string.txt"))
    config_dict["summary_report"]["from_email"] = "ptth222@uky.edu"
    config_dict["summary_report"]["to_email"] = ["ptth222@uky.edu"]
    config_dict["summary_report"]["cc_email"] = ["ptth222@uky.edu"]
    config_dict["summary_report"]["email_body"] = "Body"
    config_dict["summary_report"]["email_subject"] = "Subject"
    
    save_dir = save_and_send_reports_and_emails(authors_by_project_dict, pub_dict, config_dict, True)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_summary_report_custom_template.txt")) == read_text_from_txt(os.path.join(save_dir, "summary_report.txt"))
    assert os.path.exists(os.path.join(save_dir, "emails.json"))



def test_save_and_send_reports_and_emails_default_template(config_dict, mocker):
    
    def emails(*args, **kwargs):
        return {"creation_date":"asdf", "emails":[]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_emails_and_reports.create_project_reports_and_emails", emails)
    
    pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    authors_by_project_dict = load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))
    
    config_dict["summary_report"] = {}
    
    save_dir = save_and_send_reports_and_emails(authors_by_project_dict, pub_dict, config_dict, True)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_summary_report.txt")) == read_text_from_txt(os.path.join(save_dir, "summary_report.txt"))
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))


def test_save_and_send_reports_and_emails_collab_reports(config_dict, mocker):
    
    def emails(*args, **kwargs):
        return {"creation_date":"asdf", "emails":[]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_emails_and_reports.create_project_reports_and_emails", emails)
    
    pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    authors_by_project_dict = load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))
    
    config_dict["Authors"]["Anna Hoover"]["collaborator_report"] = {"from_email":"ptth222@uky.edu", 
                                                                    "email_body":"asdf",
                                                                    "email_subject":"asdf",
                                                                    }
    
    save_dir = save_and_send_reports_and_emails(authors_by_project_dict, pub_dict, config_dict, True)
    
    assert os.path.exists(os.path.join(save_dir, "Anna Hoover_collaborators.csv"))
    assert os.path.exists(os.path.join(save_dir, "emails.json"))
    
    
def test_save_and_send_reports_and_emails_tabular_summary(config_dict, mocker):
    
    def emails(*args, **kwargs):
        return {"creation_date":"asdf", "emails":[]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_emails_and_reports.create_project_reports_and_emails", emails)
    
    pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    authors_by_project_dict = load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["columns"] = {"Col1":"<project_name>"}
    
    save_dir = save_and_send_reports_and_emails(authors_by_project_dict, pub_dict, config_dict, True)
    
    assert os.path.exists(os.path.join(save_dir, "summary_report.csv"))
    


def test_save_and_send_reports_and_emails_manual_name(config_dict, mocker):
    
    def emails(*args, **kwargs):
        return {"creation_date":"asdf", "emails":[]}
    mocker.patch("academic_tracker.athr_srch_modularized.athr_srch_emails_and_reports.create_project_reports_and_emails", emails)
    
    pub_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
    authors_by_project_dict = load_json(os.path.join("tests", "testing_files", "authors_by_project_dict_truncated.json"))
    
    config_dict["summary_report"] = {}
    config_dict["summary_report"]["template"] = read_text_from_txt(os.path.join("tests", "testing_files", "athr_srch_build_loop_template_string.txt"))
    config_dict["summary_report"]["filename"] = "test_name.txt"
    
    save_dir = save_and_send_reports_and_emails(authors_by_project_dict, pub_dict, config_dict, True)
    
    assert os.path.exists(os.path.join(save_dir, "test_name.txt"))






@pytest.fixture(autouse=True)
def cleanup(request):
    """Cleanup a testing directory once we are finished."""
    def remove_test_dir():
        dir_contents = os.listdir()
        tracker_dirs = [folder for folder in dir_contents if re.match(r"tracker-(\d{10})", folder) or re.match(r"tracker-test-(\d{10})", folder)]
        for directory in tracker_dirs:
            shutil.rmtree(directory)
    request.addfinalizer(remove_test_dir)


# -*- coding: utf-8 -*-

import os

import pytest
import pickle
import requests

from fixtures import authors_dict
from academic_tracker.athr_srch_webio import search_PubMed_for_pubs, search_ORCID_for_pubs, search_Google_Scholar_for_pubs
from academic_tracker.athr_srch_webio import search_Crossref_for_pubs
from academic_tracker.fileio import load_json



@pytest.fixture(autouse=True)
def disable_network_calls(monkeypatch):
    def stunted_get():
        raise RuntimeError("Network access not allowed during testing!")
    monkeypatch.setattr(requests, "get", lambda *args, **kwargs: stunted_get())
    


## pymed's PubMed.query() actually returns an itertools.chain where the elements are actually the _getArticles method of PubMed, not the articles directly.
## Each call to the iterator does an http request. Here I simply put the articles in a list. 
@pytest.fixture
def pymed_query():
    with open(os.path.join("tests", "testing_files", "pymed_pubs.pkl"), "rb") as f:
        articles = pickle.load(f)
    return articles


@pytest.fixture
def expected_publication_dict():
    return {'https://doi.org/10.1016/j.chroma.2021.462426': {'pubmed_id': '34352431',
  'title': 'Direct injection analysis of per and polyfluoroalkyl substances in surface and drinking water by sample filtration and liquid chromatography-tandem mass spectrometry.',
  'abstract': 'We developed and validated a method for direct determination of per- and polyfluoroalkylated substances (PFASs) in environmental water samples without prior sample concentration. Samples are centrifuged and supernatants passed through an Acrodisc Filter (GXF/GHP 0.2\xa0\xa0um, 25\xa0\xa0mm diameter). After addition of ammonium acetate, samples are analyzed by UPLC-MS/MS using an AB Sciex 6500 plus Q-Trap mass spectrometer operated in negative multiple reaction-monitoring (MRM) mode. The instrument system incorporates a delay column between the pumps and autosampler to mitigate interference from background PFAS. The method monitors eight short-/long-chain PFAS which are identified by monitoring specific precursor product ion pairs and by their retention times and quantified using isotope mass-labeled internal standard based calibration plots. Average spiked recoveries (n\xa0=\xa08) of target analytes ranged from 84 to 110% with 4-9% relative standard deviation (RSD). The mean spiked recoveries (n\xa0=\xa08) of four surrogates were 94-106% with 3-8% RSD. For continuous calibration verification (CCV), average spiked recoveries (n\xa0=\xa08) for target analytes ranged from 88 to 114% with 4-11% RSD and for surrogates ranged from 104-112% with 3-11% RSD. The recoveries (n\xa0=\xa06) of matrix spike (MX), matrix spike duplicate (MXD), and field reagent blank (FRB) met our acceptance criteria. The limit of detection for the target analytes was between 0.007 and 0.04\xa0ng/mL. The method was used to measure PFAS in tap water and surface water.',
  'keywords': ['Acrodisc filtration',
   'Direct injection',
   'Drinking and surface water',
   'PFAS',
   'UPLC-MS/MS'],
  'journal': 'Journal of chromatography. A',
  'publication_date': {'year': 2021, 'month': 8, 'day': 6},
  'authors': [{'lastname': 'Mottaleb',
    'firstname': 'M Abdul',
    'initials': 'MA',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Division of Cardiovascular, Medicine, College of Medicine, University of Kentucky and Lexington VA Medical Center, Lexington, KY, 40536, United States. a.j.morris@uky.edu; Pressent address: Institute of Drug & Biotherapeutic Innovation, DRC, 1100 South Grand Blvd, Saint Louis University, Saint Louis, MO 63104 United States. Electronic address: m.a.mottaleb@uky.edu.'},
   {'lastname': 'Ding',
    'firstname': 'Qunxing X',
    'initials': 'QX',
    'affiliation': 'Department of Biology, College of Arts and Sciences, Kent State University, Kent, OH, 44242, United States. Electronic address: qding@kent.edu.'},
   {'lastname': 'Pennell',
    'firstname': 'Kelly G',
    'initials': 'KG',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Department of Civil Engineering, College of Engineering, University of Kentucky, Lexington KY, 40506, United States. Electronic address: kellypennell@uky.edu.',},
   {'lastname': 'Haynes',
    'firstname': 'Erin N',
    'initials': 'EN',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Department of Epidemiology, College of Public Health, University of Kentucky, Lexington KY, 40536, United States. Electronic address: erin.haynes@uky.edu.',},
   {'lastname': 'Morris',
    'firstname': 'Andrew J',
    'initials': 'AJ',
    'affiliation': 'Superfund Research Center, University of Kentucky, Lexington KY, 40506, United States; Center for Appalachian Research in Environmental Sciences, University of Kentucky, Lexington KY, 40506, United States; Division of Cardiovascular, Medicine, College of Medicine, University of Kentucky and Lexington VA Medical Center, Lexington, KY, 40536, United States. a.j.morris@uky.edu; Pressent address: Institute of Drug & Biotherapeutic Innovation, DRC, 1100 South Grand Blvd, Saint Louis University, Saint Louis, MO 63104 United States. Electronic address: a.j.morris@uky.edu.',
    'author_id': 'Andrew Morris'}],
  'methods': None,
  'conclusions': None,
  'results': None,
  'copyrights': 'Copyright © 2021. Published by Elsevier B.V.',
  'doi': '10.1016/j.chroma.2021.462426',
  'grants': ['P30 ES026529'],
  'PMCID': None}}




def test_request_pubs_from_pubmed_success(pymed_query, mocker, authors_dict, expected_publication_dict):
    def mock_query(*args, **kwargs):
        return pymed_query
    mocker.patch("academic_tracker.athr_srch_webio.pymed.PubMed.query", mock_query)
    test_publication_dict = search_PubMed_for_pubs({}, authors_dict, "ptth222@uky.edu")
    assert test_publication_dict == expected_publication_dict



@pytest.fixture
def ORCID_query():
    return load_json(os.path.join("tests", "testing_files", "ORCID_query.json"))


def test_search_ORCID_for_pubs(ORCID_query, authors_dict, mocker):
    def mock_query(*args, **kwargs):
        return ORCID_query
    mocker.patch("academic_tracker.athr_srch_webio.orcid.PublicAPI.read_record_public", mock_query)
    
    def mock_token(*args, **kwargs):
        return "sdfg"
    mocker.patch("academic_tracker.athr_srch_webio.orcid.PublicAPI.get_search_token_from_orcid", mock_token)
    
    expected_dict = load_json(os.path.join("tests", "testing_files", "ORCID_pub_dict.json"))
    
    returned_dict = search_ORCID_for_pubs({}, "qwerqwer", "asdfasdf", authors_dict)
    
    ## There is a character in the title of this publication that saves funny 
    ## so it won't be the same when you check.
    del expected_dict["https://doi.org/10.1016/j.celrep.2021.110013"]
    del returned_dict["https://doi.org/10.1016/j.celrep.2021.110013"]
         
    assert returned_dict == expected_dict



@pytest.fixture
def scholarly_pubs():
    return load_json(os.path.join("tests", "testing_files", "scholarly_pubs.json"))

        
@pytest.fixture
def scholarly_doi():
    return load_json(os.path.join("tests", "testing_files", "scholarly_DOIs.json"))


def test_search_Google_Scholar_for_pubs(authors_dict, scholarly_pubs, scholarly_doi, mocker):
    def mock_queried_author(*args, **kwargs):
        return {'container_type': 'Author',
                 'filled': ['basics'],
                 'scholar_id': '-j7fxnEAAAAJ',
                 'name': 'Andrew J. Morris',
                 'affiliation': 'Stony Brook, University of North Carolina, University of Kentucky',
                 'interests': ['Medicine'],
                 'email_domain': '@uky.edu',
                 'citedby': 23618}
    mocker.patch("academic_tracker.athr_srch_webio.scholarly.scholarly.search_author_id", mock_queried_author)
    
    def mock_fill_publications(dictionary, *args, **kwargs):
        if "sections" in kwargs:
            filled_publications = load_json(os.path.join("tests", "testing_files", "scholarly_query.json"))
            return filled_publications
        else:
            for pub in scholarly_pubs:
                if pub["author_pub_id"] == dictionary["author_pub_id"]:
                    return pub
            
    mocker.patch("academic_tracker.athr_srch_webio.scholarly.scholarly.fill", mock_fill_publications)
    
    def mock_Crossref_DOI(title, *args, **kwargs):
        if title in scholarly_doi:
            return scholarly_doi[title]
        else:
            return None
        
    mocker.patch("academic_tracker.webio.get_DOI_from_Crossref", mock_Crossref_DOI)
    
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "scholarly_pub_dict.json"))
    
    actual_pub_dict = search_Google_Scholar_for_pubs({}, authors_dict, "ptth222@uky.edu")
    
    assert actual_pub_dict == expected_pub_dict



def test_search_Crossref_for_pubs(authors_dict, mocker):
    def mock_query(*args, **kwargs):
        return load_json(os.path.join("tests", "testing_files", "Crossref_query.json"))
    mocker.patch("academic_tracker.athr_srch_webio.habanero.Crossref.works", mock_query)
       
    expected_pub_dict = load_json(os.path.join("tests", "testing_files", "Crossref_pub_dict.json"))
    
    actual_pub_dict = search_Crossref_for_pubs({}, authors_dict, "ptth222@uky.edu")
        
    assert actual_pub_dict == expected_pub_dict









# -*- coding: utf-8 -*-

import os
import requests
import copy
import re

import pytest
import shutil

from academic_tracker.__main__ import main, author_search, reference_search, PMID_reference
from academic_tracker.__main__ import find_ORCID, find_Google_Scholar, add_authors
from academic_tracker.__main__ import tokenize_reference, gen_reports_and_emails_auth, gen_reports_and_emails_ref
from academic_tracker.fileio import load_json, read_text_from_txt


@pytest.fixture(autouse=True)
def disable_network_calls(monkeypatch):
    def stunted_get():
        raise RuntimeError("Network access not allowed during testing!")
    monkeypatch.setattr(requests, "get", lambda *args, **kwargs: stunted_get())





@pytest.mark.parametrize("func_name, options", [
        
        ("author_search", ["file_path"]),
        ("reference_search", ["file_path1", "file_path2"]),
        ("find_ORCID", ["file_path"]),
        ("find_Google_Scholar", ["file_path"]),
        ("add_authors", ["file_path1", "file_path2"]),
        ("tokenize_reference", ["file_path"]),
        ("gen_reports_and_emails_auth", ["file_path1", "file_path2"]),
        ("gen_reports_and_emails_ref", ["file_path1", "file_path2", "file_path3"]),
        ])



def test_main(func_name, options, mocker, capsys):
    mocker.patch("__main__.sys.argv", ["academic_tracker", func_name] + options)
    
    def mock_call(*args, **kwargs):
        print("Call Successful")
    mocker.patch("academic_tracker.__main__." + func_name, mock_call)
    
    main()
    
    captured = capsys.readouterr()
    assert captured.out == "Call Successful" + "\n"
    


def test_main_PMID_reference(mocker, capsys):
    mocker.patch("__main__.sys.argv", ["academic_tracker", "reference_search", "file_path1", "file_path2", "--PMID_reference"])
    
    def mock_call(*args, **kwargs):
        print("Call Successful")
    mocker.patch("academic_tracker.__main__.PMID_reference" , mock_call)
    
    main()
    
    captured = capsys.readouterr()
    assert captured.out == "Call Successful" + "\n"




def test_author_search(mocker, capsys):
    def mock_call(*args, **kwargs):
        publication_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
        return publication_dict
    mocker.patch("academic_tracker.__main__.athr_srch_modularized.build_publication_dict", mock_call)
    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"), 
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--no_PubMed":False,
            "--test":True}
    
    author_search(args["<config_json_file>"], args["--no_ORCID"], 
                      args["--no_GoogleScholar"], args["--no_Crossref"], 
                      args["--no_PubMed"],
                      args["--test"], args["--prev_pub"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert not os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert os.path.exists(os.path.join(save_dir, "emails.json"))
    assert os.path.exists(os.path.join(save_dir, "publications.json"))



def test_reference_search(mocker, capsys):
    def mock_call(*args, **kwargs):
        publication_dict = load_json(os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"))
        tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
        return publication_dict, tokenized_citations
    mocker.patch("academic_tracker.__main__.ref_srch_modularized.build_publication_dict", mock_call)
    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "<references_file_or_URL>":os.path.join("tests", "testing_files", "tokenized_citations_duplicates_removed.json"),
            "--MEDLINE_reference":False,
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--no_PubMed":False,
            "--test":True}
    
    reference_search(args["<config_json_file>"], args["<references_file_or_URL>"], 
                             args["--MEDLINE_reference"], args["--no_Crossref"], 
                             args["--no_PubMed"],
                             args["--test"], args["--prev_pub"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert not os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))
    assert os.path.exists(os.path.join(save_dir, "publications.json"))



@pytest.mark.parametrize("reference_file", [
        
        ("PMID_reference.json"),
        ("PMID_reference.docx"),
        ("PMID_reference.txt"),
        ])



def test_PMID_reference(reference_file, mocker, capsys):
    def mock_call(*args, **kwargs):
        publication_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
        publication_dict["34811960"] = publication_dict["https://doi.org/10.1002/adhm.202101820"]
        publication_dict["34622577"] = publication_dict["https://doi.org/10.1002/advs.202101999"]
        del publication_dict["https://doi.org/10.1002/adhm.202101820"]
        del publication_dict["https://doi.org/10.1002/advs.202101999"]
        return publication_dict
    mocker.patch("academic_tracker.__main__.ref_srch_webio.build_pub_dict_from_PMID", mock_call)
    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "<references_file_or_URL>":os.path.join("tests", "testing_files", reference_file),
            "--MEDLINE_reference":False,
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    PMID_reference(args["<config_json_file>"], args["<references_file_or_URL>"], args["--test"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert os.path.exists(os.path.join(save_dir, "publications.json"))
    
    


@pytest.mark.parametrize("error_file, error_message", [
        
        ("PMID_reference.asdf", "Unknown file type for PMID file."),
        ("empty_file.txt", "Nothing was read from the PMID file. Make sure the file is not empty or is a supported file type."),
        ])


def test_PMID_reference_errors(error_file, error_message, mocker, capsys):
    def mock_call(*args, **kwargs):
        publication_dict = load_json(os.path.join("tests", "testing_files", "publication_dict_truncated.json"))
        publication_dict["34811960"] = publication_dict["https://doi.org/10.1002/adhm.202101820"]
        publication_dict["34622577"] = publication_dict["https://doi.org/10.1002/advs.202101999"]
        del publication_dict["https://doi.org/10.1002/adhm.202101820"]
        del publication_dict["https://doi.org/10.1002/advs.202101999"]
        return publication_dict
    mocker.patch("academic_tracker.__main__.ref_srch_webio.build_pub_dict_from_PMID", mock_call)
    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "<references_file_or_URL>":os.path.join("tests", "testing_files", error_file),
            "--MEDLINE_reference":False,
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    with pytest.raises(SystemExit):
        PMID_reference(args["<config_json_file>"], args["<references_file_or_URL>"], args["--test"])
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    


def test_find_ORCID(mocker, capsys):
    def mock_call(*args, **kwargs):
        config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        return config_dict["Authors"]
    mocker.patch("academic_tracker.__main__.webio.search_ORCID_for_ids", mock_call)
    
    def mock_config(*args, **kwargs):
        config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        del config_dict["Authors"]["Andrew Morris"]["ORCID"]
        return config_dict
    mocker.patch("academic_tracker.__main__.fileio.load_json", mock_config)
    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    find_ORCID(args["<config_json_file>"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert os.path.exists(os.path.join(save_dir, "configuration.json"))
    


def test_find_ORCID_no_new(mocker, capsys):
    def mock_call(*args, **kwargs):
        config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        return config_dict["Authors"]
    mocker.patch("academic_tracker.__main__.webio.search_ORCID_for_ids", mock_call)
        
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    find_ORCID(args["<config_json_file>"])
    
    captured = capsys.readouterr()
    
    assert captured.out.split("\n")[-2] == "No authors were matched from the ORCID results. No new file saved."
    
   

def test_find_Google_Scholar(mocker, capsys):
    def mock_call(*args, **kwargs):
        config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        return config_dict["Authors"]
    mocker.patch("academic_tracker.__main__.webio.search_Google_Scholar_for_ids", mock_call)
    
    def mock_config(*args, **kwargs):
        config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        del config_dict["Authors"]["Andrew Morris"]["scholar_id"]
        return config_dict
    mocker.patch("academic_tracker.__main__.fileio.load_json", mock_config)
    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    find_Google_Scholar(args["<config_json_file>"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert os.path.exists(os.path.join(save_dir, "configuration.json"))
    


def test_find_Google_Scholar_no_new(mocker, capsys):
    def mock_call(*args, **kwargs):
        config_dict = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
        return config_dict["Authors"]
    mocker.patch("academic_tracker.__main__.webio.search_Google_Scholar_for_ids", mock_call)
        
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    find_Google_Scholar(args["<config_json_file>"])
    
    captured = capsys.readouterr()
    
    assert captured.out.split("\n")[-2] == "No authors were matched from the Google Scholar results. No new file saved."
    
    

def test_add_authors(capsys):
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "<authors_file>":os.path.join("tests", "testing_files", "add_authors.csv"),
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    expected_config = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
    expected_config["Authors"]["Name McNamerson"] = {"first_name":"Name",
                                                     "last_name":"McNamerson",
                                                     "pubmed_name_search":"Name McNamerson",
                                                     "email":"ptth222@uky.edu",
                                                     "affiliations":["kentucky","asdf","qwr"]}
    
    add_authors(args["<config_json_file>"], args["<authors_file>"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert expected_config == load_json(os.path.join(save_dir, "configuration.json"))
    


@pytest.mark.parametrize("error_file, error_message", [
        
        ("empty_file.txt", "Unknown file type for author file."),
        ("add_authors_missing_column.csv", "Error: The following columns are required but are missing:\nlast_name"),
        ("add_authors_missing_value.csv", "Error: The following columns have null values:\nlast_name"),
        ])


def test_add_authors_errors(error_file, error_message, capsys):
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "<authors_file>":os.path.join("tests", "testing_files", error_file),
            "--verbose":True,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    expected_config = load_json(os.path.join("tests", "testing_files", "config_truncated.json"))
    expected_config["Authors"]["Name McNamerson"] = {"first_name":"Name",
                                                     "last_name":"McNamerson",
                                                     "pubmed_name_search":"Name McNamerson",
                                                     "email":"ptth222@uky.edu",
                                                     "affiliations":["kentucky","asdf","qwr"]}
    
    with pytest.raises(SystemExit):
        add_authors(args["<config_json_file>"], args["<authors_file>"])
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"




def test_tokenize_reference(mocker, capsys):
    def mock_call(*args, **kwargs):
        tokenized_citations = load_json(os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"))
        tokenized_citations.append({"authors":[], "title":"", "DOI":"", "PMID":"", "reference_line":""})
        return tokenized_citations
    mocker.patch("academic_tracker.__main__.ref_srch_webio.tokenize_reference_input", mock_call)
    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "<references_file_or_URL>":os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"),
            "--verbose":True,
            "--MEDLINE_reference":False,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    expected_text = read_text_from_txt(os.path.join("tests", "testing_files", "tokenization_report.txt"))
    
    tokenize_reference(args["<references_file_or_URL>"], args["--MEDLINE_reference"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert expected_text == read_text_from_txt(os.path.join(save_dir, "tokenization_report.txt"))




def test_gen_reports_and_emails_auth(capsys):
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated.json"),
            "<publication_json_file>":os.path.join("tests", "testing_files", "publication_dict_truncated.json"),
            "--verbose":True,
            "--MEDLINE_reference":False,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    gen_reports_and_emails_auth(args["<config_json_file>"], args["<publication_json_file>"], args["--test"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert os.path.exists(os.path.join(save_dir, "emails.json"))
    ## Should be emails.json and project reports.
    assert len(os.listdir()) > 3
    



def test_gen_reports_and_emails_ref(mocker, capsys):
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated_ref_srch_summary_report.json"),
            "<publication_json_file>":os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"),
            "<references_file_or_URL>":os.path.join("tests", "testing_files", "tokenized_citations_for_report_test.json"),
            "--verbose":True,
            "--MEDLINE_reference":False,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    gen_reports_and_emails_ref(args["<config_json_file>"], args["<references_file_or_URL>"], 
                                   args["<publication_json_file>"], args["--MEDLINE_reference"], 
                                   args["--test"], args["--prev_pub"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    
    

def test_gen_reports_and_emails_ref_no_pub_dict_keys(mocker, capsys):    
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated_ref_srch_summary_report.json"),
            "<publication_json_file>":os.path.join("tests", "testing_files", "ref_srch_gen_reports_test_pub_dict.json"),
            "<references_file_or_URL>":os.path.join("tests", "testing_files", "tokenized_citations_for_report_test2.json"),
            "--verbose":True,
            "--MEDLINE_reference":False,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    expected_report = read_text_from_txt(os.path.join("tests", "testing_files", "gen_reports_ref_summary_report.txt"))
    
    gen_reports_and_emails_ref(args["<config_json_file>"], args["<references_file_or_URL>"], 
                                   args["<publication_json_file>"], args["--MEDLINE_reference"], 
                                   args["--test"], args["--prev_pub"])
    
    captured = capsys.readouterr()
    save_dir = captured.out.strip().split(" ")[-1]
    
    assert not os.path.exists(os.path.join(save_dir, "emails.json"))
    assert os.path.exists(os.path.join(save_dir, "summary_report.txt"))
    assert expected_report == read_text_from_txt(os.path.join(save_dir, "summary_report.txt"))
    
    

def test_gen_reports_and_emails_ref_no_matches(mocker, capsys):       
    args = {"<config_json_file>":os.path.join("tests", "testing_files", "config_truncated_ref_srch_summary_report.json"),
            "<publication_json_file>":os.path.join("tests", "testing_files", "ref_srch_Crossref_pub_dict.json"),
            "<references_file_or_URL>":os.path.join("tests", "testing_files", "tokenized_citations_for_report_test_empty.json"),
            "--verbose":True,
            "--MEDLINE_reference":False,
            "--prev_pub":"ignore",
            "--no_ORCID":False,
            "--no_Crossref":False,
            "--no_GoogleScholar":False,
            "--test":True}
    
    with pytest.raises(SystemExit):    
        gen_reports_and_emails_ref(args["<config_json_file>"], args["<references_file_or_URL>"], 
                                   args["<publication_json_file>"], args["--MEDLINE_reference"], 
                                   args["--test"], args["--prev_pub"])
    
    captured = capsys.readouterr()
    assert captured.out == "Error: No entries in the publication JSON matched any reference in the provided reference." + "\n"
    
    




@pytest.fixture(autouse=True)
def cleanup(request):
    """Cleanup a testing directory once we are finished."""
    def remove_test_dir():
        dir_contents = os.listdir()
        tracker_dirs = [folder for folder in dir_contents if re.match(r"tracker-(\d{10})", folder) or re.match(r"tracker-test-(\d{10})", folder)]
        for directory in tracker_dirs:
            shutil.rmtree(directory)
    request.addfinalizer(remove_test_dir)


# -*- coding: utf-8 -*-
import pytest

import pathlib
import os
import requests
import time
import json
import subprocess
import re

import shutil


@pytest.fixture(autouse=True)
def disable_network_calls(monkeypatch):
    def stunted_get():
        raise RuntimeError("Network access not allowed during testing!")
    monkeypatch.setattr(requests, "get", lambda *args, **kwargs: stunted_get())


@pytest.fixture(scope="module", autouse=True)
def change_cwd():
    cwd = pathlib.Path.cwd()
    os.chdir(pathlib.Path("tests", "testing_files"))
    yield
    os.chdir(cwd)

@pytest.fixture(autouse=True)
def delete_folder():
    paths = [os.path.abspath(name) for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
    for path in paths:
        path = pathlib.Path(path)
        shutil.rmtree(path)
        time_to_wait=10
        time_counter = 0
        while path.exists():
            time.sleep(1)
            time_counter += 1
            if time_counter > time_to_wait:
                raise FileExistsError(path + " was not deleted within " + str(time_to_wait) + " seconds, so it is assumed that it won't be and something went wrong.")



def test_no_source_hyphens_author_search():
    """Test that the hyphen version of the no_source options work for author_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker author_search ./" + test_file  + " --no-PubMed --no-Crossref --no-ORCID --no-GoogleScholar" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert not [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "No new publications found." in output


def test_no_source_underscores_author_search():
    """Test that the underscore version of the no_source options work for author_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker author_search ./" + test_file  + " --no_PubMed --no_Crossref --no_ORCID --no_GoogleScholar" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert not [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "No new publications found." in output


def test_prev_pub_hyphen_author_search():
    """Test that the hyphen version of the prev_pub option works for author_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker author_search ./" + test_file  + " --no-PubMed --no-Crossref --no-ORCID --no-GoogleScholar --prev-pub publication_dict_truncated.json" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert not [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "No new publications found." in output
    

def test_prev_pub_underscore_author_search():
    """Test that the underscore version of the prev_pub option works for author_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker author_search ./" + test_file  + " --no-PubMed --no-Crossref --no-ORCID --no-GoogleScholar --prev_pub publication_dict_truncated.json" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert not [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "No new publications found." in output


def test_no_source_hyphen_ref_search():
    """Test that the hyphen version of the no_source options work for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " tokenized_citations_duplicates_removed.json --no-PubMed --no-Crossref" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_no_source_underscore_ref_search():
    """Test that the underscore version of the no_source options work for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " tokenized_citations_duplicates_removed.json --no_PubMed --no_Crossref" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_prev_pub_hyphen_ref_search():
    """Test that the hyphen version of the prev_pub option works for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " tokenized_citations_duplicates_removed.json --no-PubMed --no-Crossref --prev-pub publication_dict_truncated.json" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_prev_pub_underscore_ref_search():
    """Test that the underscore version of the prev_pub option works for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " tokenized_citations_duplicates_removed.json --no-PubMed --no-Crossref --prev_pub publication_dict_truncated.json" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_pmid_reference_hyphen_ref_search():
    """Test that the hyphen version of the PMID_reference option works for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " PMID_reference.json --no-PubMed --no-Crossref --PMID-reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_pmid_reference_underscore_ref_search():
    """Test that the underscore version of the PMID_reference option works for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " PMID_reference.json --no-PubMed --no-Crossref --PMID_reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_medline_reference_hyphen_ref_search():
    """Test that the hyphen version of the MEDLINE_reference option works for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " tokenized_MEDLINE2.json --no-PubMed --no-Crossref --MEDLINE-reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_medline_reference_underscore_ref_search():
    """Test that the underscore version of the MEDLINE_reference option works for reference_search."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker reference_search ./" + test_file  + " tokenized_MEDLINE2.json --no-PubMed --no-Crossref --MEDLINE_reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_medline_reference_hyphen_tokenize_ref():
    """Test that the hyphen version of the MEDLINE_reference option works for tokenize_reference."""
        
    command = "academic_tracker tokenize_reference tokenized_MEDLINE2.json --MEDLINE-reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_medline_reference_underscore_tokenize_ref():
    """Test that the underscore version of the MEDLINE_reference option works for tokenize_reference."""
        
    command = "academic_tracker tokenize_reference tokenized_MEDLINE2.json --MEDLINE_reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_medline_reference_hyphen_gen_reports_emails_ref():
    """Test that the hyphen version of the MEDLINE_reference option works for gen_reports_and_emails_ref."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker gen_reports_and_emails_ref ./" + test_file  + " tokenized_MEDLINE2.json publication_dict.json --MEDLINE-reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_medline_reference_underscore_gen_reports_emails_ref():
    """Test that the underscore version of the MEDLINE_reference option works for gen_reports_and_emails_ref."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker gen_reports_and_emails_ref ./" + test_file  + " tokenized_MEDLINE2.json publication_dict.json --MEDLINE_reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_prev_pub_hyphen_gen_reports_emails_ref():
    """Test that the hyphen version of the prev_pub option works for gen_reports_and_emails_ref."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker gen_reports_and_emails_ref ./" + test_file  + " tokenized_MEDLINE2.json publication_dict.json --prev-pub publication_dict_truncated.json --MEDLINE-reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output


def test_prev_pub_underscore_gen_reports_emails_ref():
    """Test that the underscore version of the prev_pub option works for gen_reports_and_emails_ref."""
    
    test_file = "config_truncated.json"
    
    command = "academic_tracker gen_reports_and_emails_ref ./" + test_file  + " tokenized_MEDLINE2.json publication_dict.json --prev_pub publication_dict_truncated.json --MEDLINE-reference" 
    command = command.split(" ")
    subp = subprocess.run(command, capture_output=True, encoding="UTF-8")
    output = subp.stdout

    assert [name for name in os.listdir(".") if os.path.isdir(name) and re.match(r"tracker-.*", name)]
                
    assert "Success" in output












# -*- coding: utf-8 -*-



import os
import copy

import pytest
import requests

from fixtures import  authors_dict
from academic_tracker.webio import search_ORCID_for_ids, search_Google_Scholar_for_ids
from academic_tracker.webio import get_DOI_from_Crossref, get_grants_from_Crossref
from academic_tracker.fileio import load_json


@pytest.fixture(autouse=True)
def disable_network_calls(monkeypatch):
    def stunted_get():
        raise RuntimeError("Network access not allowed during testing!")
    monkeypatch.setattr(requests, "get", lambda *args, **kwargs: stunted_get())



@pytest.fixture
def ORCID_query():
    return load_json(os.path.join("tests", "testing_files", "ORCID_author_search_query.json"))


def test_search_ORCID_for_ids_already_has_id(ORCID_query, authors_dict, mocker):
    def mock_query(*args, **kwargs):
        return ORCID_query
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.search", mock_query)
    
    def mock_token(*args, **kwargs):
        return "sdfg"
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.get_search_token_from_orcid", mock_token)
    
    authors_dict_check = copy.deepcopy(authors_dict)
    
    assert search_ORCID_for_ids("qwerqwer", "asdfasdf", authors_dict) == authors_dict_check
    

def test_search_ORCID_for_ids_no_affiliations(ORCID_query, authors_dict, mocker):
    def mock_query(*args, **kwargs):
        return ORCID_query
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.search", mock_query)
    
    def mock_token(*args, **kwargs):
        return "sdfg"
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.get_search_token_from_orcid", mock_token)
    
    del authors_dict["Andrew Morris"]["affiliations"]
    authors_dict["Andrew Morris"]["ORCID"] == ""
    
    authors_dict_check = copy.deepcopy(authors_dict)
    
    assert search_ORCID_for_ids("qwerqwer", "asdfasdf", authors_dict) == authors_dict_check
    
    
def test_search_ORCID_for_ids_not_found(ORCID_query, authors_dict, mocker):
    def mock_query(*args, **kwargs):
        return ORCID_query
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.search", mock_query)
    
    def mock_token(*args, **kwargs):
        return "sdfg"
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.get_search_token_from_orcid", mock_token)
    
    authors_dict["Andrew Morris"]["ORCID"] == ""
    
    authors_dict_check = copy.deepcopy(authors_dict)
    
    assert search_ORCID_for_ids("qwerqwer", "asdfasdf", authors_dict) == authors_dict_check
    
    
def test_search_ORCID_for_ids_found(ORCID_query, authors_dict, mocker):
    def mock_query(*args, **kwargs):
        return ORCID_query
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.search", mock_query)
    
    def mock_token(*args, **kwargs):
        return "sdfg"
    mocker.patch("academic_tracker.webio.orcid.PublicAPI.get_search_token_from_orcid", mock_token)
    
    authors_dict["Andrew Morris"]["ORCID"] == ""
    authors_dict["Andrew Morris"]["affiliations"] == ["Bristol"]
    
    authors_dict_check = copy.deepcopy(authors_dict)
    authors_dict_check["Andrew Morris"]["ORCID"] = "0000-0003-1910-4865"
    
    assert search_ORCID_for_ids("qwerqwer", "asdfasdf", authors_dict) == authors_dict_check




@pytest.fixture
def scholarly_authors():
    return load_json(os.path.join("tests", "testing_files", "scholarly_author_query.json"))


def test_search_Google_Scholar_for_ids_already_has_id(scholarly_authors, authors_dict, mocker):
    def mock_queried_author(*args, **kwargs):
        return scholarly_authors
    mocker.patch("academic_tracker.webio.scholarly.scholarly.search_author", mock_queried_author)
    
    authors_dict_check = copy.deepcopy(authors_dict)
    
    assert search_Google_Scholar_for_ids(authors_dict) == authors_dict_check
    
    
def test_search_Google_Scholar_for_ids_no_affiliations(scholarly_authors, authors_dict, mocker):
    def mock_queried_author(*args, **kwargs):
        return scholarly_authors
    mocker.patch("academic_tracker.webio.scholarly.scholarly.search_author", mock_queried_author)
    
    del authors_dict["Andrew Morris"]["affiliations"]
    authors_dict["Andrew Morris"]["scholar_id"] == ""
    
    authors_dict_check = copy.deepcopy(authors_dict)
    
    assert search_Google_Scholar_for_ids(authors_dict) == authors_dict_check
    
    
def test_search_Google_Scholar_for_ids_not_found(scholarly_authors, authors_dict, mocker):
    def mock_queried_author(*args, **kwargs):
        return scholarly_authors
    mocker.patch("academic_tracker.webio.scholarly.scholarly.search_author", mock_queried_author)
    
    authors_dict["Andrew Morris"]["scholar_id"] == ""
    authors_dict["Andrew Morris"]["first_name"] == "asdf"
    
    authors_dict_check = copy.deepcopy(authors_dict)
    authors_dict_check["Andrew Morris"]["scholar_id"] == "-j7fxnEAAAAJ"
    
    assert search_Google_Scholar_for_ids(authors_dict) == authors_dict_check
    
    
def test_search_Google_Scholar_for_ids_found(scholarly_authors, authors_dict, mocker):
    def mock_queried_author(*args, **kwargs):
        return scholarly_authors
    mocker.patch("academic_tracker.webio.scholarly.scholarly.search_author", mock_queried_author)
    
    authors_dict["Andrew Morris"]["scholar_id"] == ""
    
    authors_dict_check = copy.deepcopy(authors_dict)
    
    assert search_Google_Scholar_for_ids(authors_dict) == authors_dict_check
    
    
    


def test_get_DOI_from_Crossref_DOI_found(mocker):
    def mock_query(*args, **kwargs):
        return load_json(os.path.join("tests", "testing_files", "Crossref_DOI_query.json"))
    mocker.patch("academic_tracker.webio.habanero.Crossref.works", mock_query)
    
    assert get_DOI_from_Crossref("The Existential Dimension to Aging", "ptth222@uky.edu") == '10.1353/pbm.2020.0014'


def test_get_DOI_from_Crossref_DOI_not_found(mocker):
    def mock_query(*args, **kwargs):
        return load_json(os.path.join("tests", "testing_files", "Crossref_DOI_query.json"))
    mocker.patch("academic_tracker.webio.habanero.Crossref.works", mock_query)
    
    assert get_DOI_from_Crossref("asdfasdf", "ptth222@uky.edu") == None


def test_get_grants_from_Crossref_grants_found(mocker):
    def mock_query(*args, **kwargs):
        return load_json(os.path.join("tests", "testing_files", "Crossref_grant_query.json"))
    mocker.patch("academic_tracker.webio.habanero.Crossref.works", mock_query)
    
    assert get_grants_from_Crossref("Multifunctional temperature\u2010responsive polymers as advanced biomaterials and beyond", "ptth222@uky.edu", ['P42ES007380']) == ['P42ES007380']


def test_get_grants_from_Crossref_grants_not_found(mocker):
    def mock_query(*args, **kwargs):
        return load_json(os.path.join("tests", "testing_files", "Crossref_grant_query.json"))
    mocker.patch("academic_tracker.webio.habanero.Crossref.works", mock_query)
    
    assert get_grants_from_Crossref("asdfasdf", "ptth222@uky.edu", ['P42ES007380']) == None












# -*- coding: utf-8 -*-


import pytest
from jsonschema import FormatChecker, ValidationError
from contextlib import nullcontext as does_not_raise

from academic_tracker.user_input_checking import tracker_validate, cli_inputs_check, config_file_check, config_report_check
from academic_tracker.user_input_checking import prev_pubs_file_check, ref_config_file_check, tok_reference_check
from fixtures import passing_config


@pytest.fixture
def empty_args():
    return {"--verbose":False,
            "--test":False,
            "--prev_pub":None,
            '--MEDLINE_reference': False,
            '--PMID_reference': False,
            '--no_Crossref': False,
            '--no_GoogleScholar': False,
            '--no_ORCID': False,}


## Commenting $schema out because the jsonschema package produces warnings if left in. It is a known issue in their package. 10-18-2021
@pytest.fixture
def test_schema():
    schema = {
#     "$schema": "https://json-schema.org/draft/2020-12/schema",
     "title": "Test Schema",
     "description": "Schema to test tracker_validate",
     
     "type": "object",
     "minProperties":1,
     "properties": {
             "required_test":{"type": "object",
                              "properties": {"required_test": {"type": "string"}},
                              "required": ["required_test"]},
             "empty_string_test": {"type": "string", "minLength": 1},
             "empty_list_test": {"type": "array", "minItems":1},
             "wrong_type_test": {"type": "string"}, 
             "wrong_format_test": {"type": "string", "format": "email"},
             "wrong_pattern_test": {"type": "string", "pattern": "^asdf$"},
             "other_error_type": {"type": "number", "exclusiveMaximum":100}
             },
     "required": ["required_test"]
             
    }
     
    return schema
 


@pytest.mark.parametrize("instance, error_message", [
        
        ({}, "ValidationError: An error was found in the Test Schema. \nThe Test Schema cannot be empty."),
        ({"asdf":"asdf"}, "ValidationError: An error was found in the Test Schema. \nThe required property \'required_test\' is missing."),
        ({"required_test":{"asdf":"asdf"}}, "ValidationError: An error was found in the Test Schema. \nThe entry [\'required_test\'] is missing the required property \'required_test\'."),
        ({"required_test":{"required_test":""}, "empty_string_test":""}, "ValidationError: An error was found in the Test Schema. \nThe value for ['empty_string_test'] cannot be an empty string."),
        ({"required_test":{"required_test":""}, "empty_list_test":[]}, "ValidationError: An error was found in the Test Schema. \nThe value for ['empty_list_test'] cannot be empty."),
        ({"required_test":{"required_test":""}, "wrong_type_test":123}, "ValidationError: An error was found in the Test Schema. \nThe value for ['wrong_type_test'] is not of type \"string\"."),
        ({"required_test":{"required_test":""}, "wrong_format_test":"asdf"}, "ValidationError: An error was found in the Test Schema. \nThe value for ['wrong_format_test'] is not a valid email."),
        ({"required_test":{"required_test":""}, "wrong_pattern_test":"qwer"}, "ValidationError: An error was found in the Test Schema. \nThe value for ['wrong_pattern_test'] did not fit the pattern."),
        ])


def test_tracker_validate_custom_messages(instance, test_schema, error_message, capsys):
    
    pattern_messages = {"wrong_pattern_test": " did not fit the pattern."}
    
    with pytest.raises(SystemExit):
        tracker_validate(instance, test_schema, pattern_messages, format_checker=FormatChecker())
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"



def test_tracker_validate_other_errors(test_schema, capsys):
    
    instance = {"required_test":{"required_test":""}, "other_error_type":1000}
    
    with pytest.raises(ValidationError):
        tracker_validate(instance, test_schema)
        

def test_tracker_validate_no_error(test_schema):
    with does_not_raise():
        tracker_validate({"required_test":{"required_test":""}}, test_schema)



@pytest.mark.parametrize("args", [
        ({"--prev_pub":""}),
        ])


def test_cli_inputs_check_errors(empty_args, args):
    
    empty_args.update(args)
    
    with pytest.raises(SystemExit):
        cli_inputs_check(empty_args)
        

def test_cli_inputs_check_no_error(empty_args):
    with does_not_raise():
        cli_inputs_check(empty_args)
        
        

#######################
## Author Config
#######################   
@pytest.mark.parametrize("config_project_errors", [
        ({"grants":""}),        ##type
        ({"grants":[]}),        ##minItems
        ({"grants":[123]}),     ##item type
        ({"grants":[""]}),      ##item minLength
        ({"cutoff_year":"123"}),
        ({"cutoff_year":1}),
        ({"cutoff_year":99999}),
        ({"affiliations":""}),        ##type
        ({"affiliations":[]}),        ##minItems
        ({"affiliations":[123]}),     ##item type
        ({"affiliations":[""]}),      ##item minLength
        ({"project_report":123}),
        ({"project_report":{"template":"asdf", ## dependency No email_subject
                            "from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"project_report":{"template":"asdf",  ## dependency No email_body
                            "from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"project_report":{"template":"asdf",   ## dependency No from_email
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",
                            "email_body":"asdf"}}),
        ({"project_report":{"template":"asdf",  ## dependency No email_subject
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"project_report":{"template":"asdf",  ## dependency No email_body
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"collaborator_report":123}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"collaborator_report":{"to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",
                            "email_body":"asdf"}}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"authors":""}),        ##type
        ({"authors":[]}),        ##minItems
        ({"authors":[123]}),     ##item type
        ({"authors":[""]}),      ##item minLength
        ])
 
 
def test_config_file_project_check_errors(passing_config, config_project_errors):
    passing_config["project_descriptions"]["project 1"].update(config_project_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)
        



@pytest.mark.parametrize("project_report_errors", [
        ({"from_email":123}),
        ({"from_email":"asdf"}),    ## format
        ({"cc_email":"asdf"}),
        ({"cc_email":["asdf", "email@email.com"]}),
        ({"to_email":"asdf"}),
        ({"to_email":["asdf", "email@email.com"]}),
        ({"columns":123}),
        ({"columns":{}}),
        ({"columns":{"asdf":""}}),
        ({"sort":123}),
        ({"sort":[]}),
        ({"separator":123}),
        ({"separator":""}),
        ({"separator":"asdf"}),
        ({"column_order":123}),
        ({"column_order":[]}),
        ({"file_format":123}),
        ({"file_format":"asdf"}),
        ({"filename":123}),
        ({"filename":""}),
        ({"template":123}),
        ({"template":""}),
        ({"email_subject":123}),
        ({"email_subject":""}),
        ({"email_body":123}),
        ({"email_body":""}),
        ])
 
 
def test_config_file_project_report_attributes_check_errors(passing_config, project_report_errors):
    passing_config["project_descriptions"]["project 1"]["project_report"].update(project_report_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)
        

def test_config_file_project_report_sort_error_project(passing_config, capsys):
    passing_config["project_descriptions"]["project 1"]["project_report"] = {"columns":{"Name":"asdf"},
                                                                                  "sort":["asdf"]}
    error_message = "ValidationError: The \"sort\" attribute for the project_report in project project 1 " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"sort\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_project_report_column_order_error_project(passing_config, capsys):
    passing_config["project_descriptions"]["project 1"]["project_report"] = {"columns":{"Name":"asdf"},
                                                                                  "column_order":["asdf"]}
    error_message = "ValidationError: The \"column_order\" attribute for the project_report in project project 1 " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"column_order\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_project_report_column_order_error_not_all_names_project(passing_config, capsys):
    passing_config["project_descriptions"]["project 1"]["project_report"] = {"columns":{"Name":"asdf",
                                                                                        "Title":"asdf"},
                                                                                  "column_order":["Name"]}
    error_message = "ValidationError: The \"column_order\" attribute for the project_report in project project 1 " +\
                     "does not have all of the column names in \"columns\". Every column in \"columns\" must be in \"column_order\"."
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    
    
def test_config_file_project_report_sort_error_author(passing_config, capsys):
    passing_config["Authors"]["Andrew Morris"]["project_report"] = {"columns":{"Name":"asdf"},
                                                                         "sort":["asdf"]}
    error_message = "ValidationError: The \"sort\" attribute for the project_report for author Andrew Morris " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"sort\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_project_report_column_order_error_author(passing_config, capsys):
    passing_config["Authors"]["Andrew Morris"]["project_report"] = {"columns":{"Name":"asdf"},
                                                                         "column_order":["asdf"]}
    error_message = "ValidationError: The \"column_order\" attribute for the project_report for author Andrew Morris " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"column_order\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_project_report_column_order_error_not_all_names_author(passing_config, capsys):
    passing_config["Authors"]["Andrew Morris"]["project_report"] = {"columns":{"Name":"asdf",
                                                                               "Title":"asdf"},
                                                                                  "column_order":["Name"]}
    error_message = "ValidationError: The \"column_order\" attribute for the project_report for author Andrew Morris " +\
                     "does not have all of the column names in \"columns\". Every column in \"columns\" must be in \"column_order\"."
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
        


@pytest.mark.parametrize("collaborator_report_errors", [
        ({"from_email":123}),
        ({"from_email":"asdf"}),    ## format
        ({"cc_email":"asdf"}),
        ({"cc_email":["asdf", "email@email.com"]}),
        ({"to_email":"asdf"}),
        ({"to_email":["asdf", "email@email.com"]}),
        ({"columns":123}),
        ({"columns":{}}),
        ({"columns":{"asdf":""}}),
        ({"sort":123}),
        ({"sort":[]}),
        ({"separator":123}),
        ({"separator":""}),
        ({"separator":"asdf"}),
        ({"column_order":123}),
        ({"column_order":[]}),
        ({"file_format":123}),
        ({"file_format":"asdf"}),
        ({"filename":123}),
        ({"filename":""}),
        ({"template":123}),
        ({"template":""}),
        ({"email_subject":123}),
        ({"email_subject":""}),
        ({"email_body":123}),
        ({"email_body":""}),
        ])
 
 
def test_config_file_collaborator_report_attributes_check_errors(passing_config, collaborator_report_errors):
    passing_config["project_descriptions"]["project 1"]["collaborator_report"].update(collaborator_report_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)
        
        
def test_config_file_collaborator_report_sort_error_project(passing_config, capsys):
    passing_config["project_descriptions"]["project 1"]["collaborator_report"] = {"columns":{"Name":"asdf"},
                                                                                  "sort":["asdf"]}
    error_message = "ValidationError: The \"sort\" attribute for the collaborator_report in project project 1 " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"sort\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_collaborator_report_column_order_error_project(passing_config, capsys):
    passing_config["project_descriptions"]["project 1"]["collaborator_report"] = {"columns":{"Name":"asdf"},
                                                                                  "column_order":["asdf"]}
    error_message = "ValidationError: The \"column_order\" attribute for the collaborator_report in project project 1 " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"column_order\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_collaborator_report_column_order_error_not_all_names_project(passing_config, capsys):
    passing_config["project_descriptions"]["project 1"]["collaborator_report"] = {"columns":{"Name":"asdf",
                                                                                        "Title":"asdf"},
                                                                                  "column_order":["Name"]}
    error_message = "ValidationError: The \"column_order\" attribute for the collaborator_report in project project 1 " +\
                     "does not have all of the column names in \"columns\". Every column in \"columns\" must be in \"column_order\"."
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    
    
def test_config_file_collaborator_report_sort_error_author(passing_config, capsys):
    passing_config["Authors"]["Andrew Morris"]["collaborator_report"] = {"columns":{"Name":"asdf"},
                                                                         "sort":["asdf"]}
    error_message = "ValidationError: The \"sort\" attribute for the collaborator_report for author Andrew Morris " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"sort\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_collaborator_report_column_order_error_author(passing_config, capsys):
    passing_config["Authors"]["Andrew Morris"]["collaborator_report"] = {"columns":{"Name":"asdf"},
                                                                         "column_order":["asdf"]}
    error_message = "ValidationError: The \"column_order\" attribute for the collaborator_report for author Andrew Morris " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"column_order\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_collaborator_report_column_order_error_not_all_names_author(passing_config, capsys):
    passing_config["Authors"]["Andrew Morris"]["collaborator_report"] = {"columns":{"Name":"asdf",
                                                                               "Title":"asdf"},
                                                                                  "column_order":["Name"]}
    error_message = "ValidationError: The \"column_order\" attribute for the collaborator_report for author Andrew Morris " +\
                     "does not have all of the column names in \"columns\". Every column in \"columns\" must be in \"column_order\"."
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
        


@pytest.mark.parametrize("summary_report_errors", [
        ({"from_email":123}),
        ({"from_email":"asdf"}),    ## format
        ({"cc_email":"asdf"}),
        ({"cc_email":["asdf", "email@email.com"]}),
        ({"to_email":"asdf"}),
        ({"to_email":["asdf", "email@email.com"]}),
        ({"columns":123}),
        ({"columns":{}}),
        ({"columns":{"asdf":""}}),
        ({"sort":123}),
        ({"sort":[]}),
        ({"separator":123}),
        ({"separator":""}),
        ({"separator":"asdf"}),
        ({"column_order":123}),
        ({"column_order":[]}),
        ({"file_format":123}),
        ({"file_format":"asdf"}),
        ({"filename":123}),
        ({"filename":""}),
        ({"template":123}),
        ({"template":""}),
        ({"email_subject":123}),
        ({"email_subject":""}),
        ({"email_body":123}),
        ({"email_body":""}),
        ])
    
def test_config_file_summary_report_attributes_check_errors(passing_config, summary_report_errors):
    passing_config["summary_report"].update(summary_report_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)
        

def test_config_file_summary_report_sort_error_project(passing_config, capsys):
    passing_config["summary_report"] = {"columns":{"Name":"asdf"},
                                        "sort":["asdf"]}
    error_message = "ValidationError: The \"sort\" attribute for the summary_report " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"sort\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_summary_report_column_order_error_project(passing_config, capsys):
    passing_config["summary_report"] = {"columns":{"Name":"asdf"},
                                        "column_order":["asdf"]}
    error_message = "ValidationError: The \"column_order\" attribute for the summary_report " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"column_order\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_config_file_summary_report_column_order_error_not_all_names_project(passing_config, capsys):
    passing_config["summary_report"] = {"columns":{"Name":"asdf",
                                                   "Title":"asdf"},
                                                   "column_order":["Name"]}
    error_message = "ValidationError: The \"column_order\" attribute for the summary_report " +\
                     "does not have all of the column names in \"columns\". Every column in \"columns\" must be in \"column_order\"."
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"



@pytest.mark.parametrize("summary_report_errors", [
        ({"summary_report":123}),
        ({"summary_report":{"template":"asdf",  ## dependency No to_email
                            "from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",
                            "email_body":"asdf"}}),
        ({"summary_report":{"template":"asdf",  ## dependency No email_subject
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"summary_report":{"template":"asdf",  ## dependency No email_body
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ])
    
def test_config_file_summary_report_check_errors(passing_config, summary_report_errors):
    passing_config.update(summary_report_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)



@pytest.mark.parametrize("config_ORCID_errors", [
        ({"ORCID_key":123}),
        ({"ORCID_key":""}),
        ({"ORCID_secret":123}),
        ({"ORCID_secret":""}),
        ])
 
 
def test_config_file_ORCID_check_errors(passing_config, config_ORCID_errors):
    passing_config["ORCID_search"].update(config_ORCID_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)



@pytest.mark.parametrize("config_PubMed_errors", [
        ({"PubMed_email":123}),
        ({"PubMed_email":"asdf"}),
        ])
 
 
def test_config_file_PubMed_check_errors(passing_config, config_PubMed_errors):
    passing_config["PubMed_search"].update(config_PubMed_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)
        

@pytest.mark.parametrize("config_Crossref_errors", [
        ({"mailto_email":123}),
        ({"mailto_email":"asdf"}),
        ])
 
 
def test_config_file_Crossref_check_errors(passing_config, config_Crossref_errors):
    passing_config["Crossref_search"].update(config_Crossref_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)


def test_config_file_check_no_error(passing_config):
    with does_not_raise():
        config_file_check(passing_config, False, False, False, False)


def test_config_file_check_empty_file_error():
    with pytest.raises(SystemExit):
        config_file_check({}, False, False, False, False)

def test_config_file_check_missing_required_error(passing_config):
    del passing_config["project_descriptions"]
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)
        
        
    
@pytest.mark.parametrize("author_errors", [
        ({"grants":""}),        ##type
        ({"grants":[]}),        ##minItems
        ({"grants":[123]}),     ##item type
        ({"grants":[""]}),      ##item minLength
        ({"cutoff_year":"123"}),
        ({"cutoff_year":1}),
        ({"cutoff_year":99999}),
        ({"affiliations":""}),        ##type
        ({"affiliations":[]}),        ##minItems
        ({"affiliations":[123]}),     ##item type
        ({"affiliations":[""]}),      ##item minLength
        ({"project_report":123}),
        ({"project_report":{"template":"asdf", ## dependency No email_subject
                            "from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"project_report":{"template":"asdf",  ## dependency No email_body
                            "from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"project_report":{"template":"asdf",   ## dependency No from_email
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",
                            "email_body":"asdf"}}),
        ({"project_report":{"template":"asdf",  ## dependency No email_subject
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"project_report":{"template":"asdf",  ## dependency No email_body
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"collaborator_report":123}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"collaborator_report":{"to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",
                            "email_body":"asdf"}}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"collaborator_report":{"from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ({"first_name":123}),
        ({"first_name":""}),
        ({"last_name":123}),
        ({"last_name":""}),
        ({"pubmed_name_search":123}),
        ({"pubmed_name_search":""}),
        ({"email":123}),
        ({"email":"asdf"}),
        ({"ORCID":123}),
        ({"ORCID":"asdf"}),    ##pattern
        ])
 
 
def test_author_file_check_errors(passing_config, author_errors):
    passing_config["Authors"]["Andrew Morris"].update(author_errors)
    with pytest.raises(SystemExit):
        config_file_check(passing_config, False, False, False, False)
        


def test_config_file_check_schema_reduction(passing_config):
    del passing_config["ORCID_search"]["ORCID_key"]
    del passing_config["Crossref_search"]
    del passing_config["PubMed_search"]
    with does_not_raise():
        config_file_check(passing_config, True, True, True, True)




#######################
## Reference Config
#######################
@pytest.mark.parametrize("summary_report_errors", [
        ({"from_email":123}),
        ({"from_email":"asdf"}),    ## format
        ({"cc_email":"asdf"}),
        ({"cc_email":["asdf", "email@email.com"]}),
        ({"to_email":"asdf"}),
        ({"to_email":["asdf", "email@email.com"]}),
        ({"template":123}),
        ({"template":""}),
        ({"email_subject":123}),
        ({"email_subject":""}),
        ({"email_body":123}),
        ({"email_body":""}),
        ])
    
def test_ref_config_file_summary_report_attributes_check_errors(passing_config, summary_report_errors):
    passing_config["summary_report"].update(summary_report_errors)
    with pytest.raises(SystemExit):
        ref_config_file_check(passing_config, False, False)
        

def test_ref_config_file_summary_report_sort_error_project(passing_config, capsys):
    passing_config["summary_report"] = {"columns":{"Name":"asdf"},
                                        "sort":["asdf"]}
    error_message = "ValidationError: The \"sort\" attribute for the summary_report " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"sort\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"
    

def test_ref_config_file_summary_report_column_order_error_project(passing_config, capsys):
    passing_config["summary_report"] = {"columns":{"Name":"asdf"},
                                        "column_order":["asdf"]}
    error_message = "ValidationError: The \"column_order\" attribute for the summary_report " +\
                     "has values that are not column names in \"columns\".\nThe following names in \"column_order\" " +\
                     "could not be matched to a column in \"columns\":\n\nasdf"
    with pytest.raises(SystemExit):
        config_report_check(passing_config)
    captured = capsys.readouterr()
    assert captured.out == error_message + "\n"



@pytest.mark.parametrize("summary_report_errors", [
        ({"summary_report":123}),
        ({"summary_report":{"template":"asdf",  ## dependency No to_email
                            "from_email":"email@email.com", 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",
                            "email_body":"asdf"}}),
        ({"summary_report":{"template":"asdf",  ## dependency No email_subject
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_body":"asdf"}}),
        ({"summary_report":{"template":"asdf",  ## dependency No email_body
                            "from_email":"email@email.com", 
                            "to_email":["email@email.com"], 
                            "cc_email":["email@email.com"],
                            "email_subject":"asdf",}}),
        ])
    
def test_ref_config_file_summary_report_check_errors(passing_config, summary_report_errors):
    passing_config.update(summary_report_errors)
    with pytest.raises(SystemExit):
        ref_config_file_check(passing_config, False, False)



@pytest.mark.parametrize("config_PubMed_errors", [
        ({"PubMed_email":123}),
        ({"PubMed_email":"asdf"}),
        ])
 
 
def test_ref_config_file_PubMed_check_errors(passing_config, config_PubMed_errors):
    passing_config["PubMed_search"].update(config_PubMed_errors)
    with pytest.raises(SystemExit):
        ref_config_file_check(passing_config, False, False)
        

@pytest.mark.parametrize("config_Crossref_errors", [
        ({"mailto_email":123}),
        ({"mailto_email":"asdf"}),
        ])
 
 
def test_ref_config_file_Crossref_check_errors(passing_config, config_Crossref_errors):
    passing_config["Crossref_search"].update(config_Crossref_errors)
    with pytest.raises(SystemExit):
        ref_config_file_check(passing_config, False, False)


def test_ref_config_file_check_no_error(passing_config):
    with does_not_raise():
        ref_config_file_check(passing_config, False, False)


def test_ref_config_file_check_empty_file_error():
    with pytest.raises(SystemExit):
        ref_config_file_check({}, False, False)

def test_ref_config_file_check_missing_required_error(passing_config):
    del passing_config["PubMed_search"]
    with pytest.raises(SystemExit):
        ref_config_file_check(passing_config, False, False)


def test_ref_config_file_check_schema_reduction(passing_config):
    del passing_config["Crossref_search"]
    del passing_config["PubMed_search"]
    with does_not_raise():
        ref_config_file_check(passing_config, True, True)




@pytest.fixture
def passing_pubs():
    return {"30602735" : {
            'abstract': "Following the publication of this article the authors noted that Torfi Sigurdsson's name was misspelled. Instead of Sigrudsson it should be Sigurdsson. The PDF and HTML versions of the paper have been modified accordingly.\xa0The authors would like to apologise for this error and the inconvenience this may have caused.",
             'authors': [
                     { 'affiliation': 'Department of Neurology, University Medical Center, Johannes Gutenberg-University, Mainz, Germany.',
                       'firstname': 'Carine',
                       'initials': 'C',
                       'lastname': 'Thalman'},],
             'conclusions': None,
             'copyrights': None,
             'doi': '10.1038/s41380-018-0320-1',
             'journal': 'Molecular psychiatry',
             'keywords': [],
             'methods': None,
             'publication_date': {"year":2019, "month":1, "day":4},
             'pubmed_id': '30602735',
             'results': None,
             'title': 'Correction: Synaptic phospholipids as a new target for cortical hyperexcitability and E/I balance in psychiatric disorders.',
             'grants':['P42 ES007380', 'P42 ES007381'],
             'PMCID':'PMC7933073'}
 }
    
@pytest.mark.parametrize("pub_errors", [
        ({"abstract":123}),
        ({"authors":123}),
        ({"authors":[]}),
        ({"authors":[{"affiliation":"asdf", "firstname":"asdf", "initials":"asdf", "lastname":123}]}),
        ({"authors":[{"affiliation":"asdf", "firstname":"asdf", "initials":123, "lastname":"asdf"}]}),
        ({"authors":[{"affiliation":"asdf", "firstname":123, "initials":"asdf", "lastname":"asdf"}]}),
        ({"authors":[{"affiliation":123, "firstname":"asdf", "initials":"asdf", "lastname":"asdf"}]}),
        ({"authors":[{"firstname":"asdf", "initials":"asdf", "lastname":"asdf"}]}),
        ({"authors":[{"affiliation":"asdf", "initials":"asdf", "lastname":"asdf"}]}),
        ({"authors":[{"affiliation":"asdf", "firstname":"adf", "lastname":"asdf"}]}),
        ({"authors":[{"affiliation":"asdf", "firstname":"asdf", "initials":"asdf"}]}),
        ({"conclusions":123}),
        ({"copyrights":123}),
        ({"doi":123}),
        ({"journal":123}),
        ({"keywords":123}),
        ({"keywords":[123]}),
        ({"methods":123}),
        ({"publication_date":123}),
        ({"publication_date":{"year":"asdf", "month":1, "day":1}}),
        ({"publication_date":{"year":1, "month":"asdf", "day":1}}),
        ({"publication_date":{"year":1, "month":1, "day":"asdf"}}),
        ({"publication_date":{"year":1, "month":1, }}),
        ({"publication_date":{"year":1, "day":1}}),
        ({"publication_date":{"month":1, "day":1}}),
        ({"pubmed_id":123}),
        ({"results":123}),
        ({"title":123}),
        ({"grants":123}),
        ({"grants":[123]}),
        ({"PMCID":123}),
        ])
 
 
def test_prev_pubs_file_check_errors(passing_pubs, pub_errors):
    passing_pubs.update(pub_errors)
    with pytest.raises(SystemExit):
        prev_pubs_file_check(passing_pubs)


def test_prev_pubs_file_check_no_error(passing_pubs):
    with does_not_raise():
        prev_pubs_file_check(passing_pubs)


def test_prev_pubs_file_check_missing_required_error(passing_pubs):
    del passing_pubs["30602735"]["abstract"]
    with pytest.raises(SystemExit):
        prev_pubs_file_check(passing_pubs)




@pytest.fixture
def passing_tok():
    return [{"authors": [{"last": "Last", "initials": "", "first": "First", "middle": ""}],
           "title": "Pub Title",
           "PMID": "",
           "DOI": "",
           "reference_line": "", 
           "pub_dict_key": ""}]

@pytest.mark.parametrize("tok_errors", [
        ({"authors":123}),
        ({"authors":[{"last":"asdf", "first":"asdf", "initials":"asdf", "middle":123}]}),
        ({"authors":[{"last":"asdf", "first":"asdf", "initials":123, "middle":"asdf"}]}),
        ({"authors":[{"last":"asdf", "first":123, "initials":"asdf", "middle":"asdf"}]}),
        ({"authors":[{"last":123, "first":"asdf", "initials":"asdf", "middle":"asdf"}]}),
        ({"authors":[{"first":"asdf", "initials":"asdf", "middle":"asdf"}]}),
        ({"title":123}),
        ({"PMID":123}),
        ({"DOI":123}),
        ({"reference_line":123}),
        ({"pub_dict_key":123}),
        ])


def test_tok_reference_check_errors(passing_tok, tok_errors):
    passing_tok[0].update(tok_errors)
    with pytest.raises(SystemExit):
        tok_reference_check(passing_tok)
    
    
def test_tok_reference_check_no_error(passing_tok):
    with does_not_raise():
        tok_reference_check(passing_tok)
    
    
def test_tok_reference_check_missing_required_error(passing_tok):
    del passing_tok[0]["title"]
    with pytest.raises(SystemExit):
        tok_reference_check(passing_tok)








